# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class DriverEfficiencyRawDataObjectResponseBody(UniversalBaseModel):
    """
    Driver Efficiency raw data. This object is returned when the “raw” format is specified in “dataFormats”.
    """

    anticipation_brake_event_count: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="anticipationBrakeEventCount"),
        pydantic.Field(
            alias="anticipationBrakeEventCount",
            description="Total number of quick braking events (less than one second after accelerating).",
        ),
    ] = None
    average_vehicle_weight_kg: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="averageVehicleWeightKg"),
        pydantic.Field(alias="averageVehicleWeightKg", description="Average vehicle weight in kilograms."),
    ] = None
    coasting_duration_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="coastingDurationMs"),
        pydantic.Field(
            alias="coastingDurationMs",
            description="Time spent without engaging the accelerator or brake in milliseconds.",
        ),
    ] = None
    cruise_control_duration_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="cruiseControlDurationMs"),
        pydantic.Field(alias="cruiseControlDurationMs", description="Time spent in cruise control in milliseconds."),
    ] = None
    drive_time_duration_ms: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="driveTimeDurationMs"),
        pydantic.Field(alias="driveTimeDurationMs", description="Total driving time in milliseconds."),
    ]
    engine_on_duration_ms: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="engineOnDurationMs"),
        pydantic.Field(alias="engineOnDurationMs", description="Total engine-on time in milliseconds."),
    ]
    green_band_duration_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="greenBandDurationMs"),
        pydantic.Field(
            alias="greenBandDurationMs",
            description="Time spent driving within the configurable green band in milliseconds.",
        ),
    ] = None
    high_grade_road_driving_duration_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="highGradeRoadDrivingDurationMs"),
        pydantic.Field(
            alias="highGradeRoadDrivingDurationMs",
            description="Time spent driving on high-grade roads in milliseconds.",
        ),
    ] = None
    high_torque_duration_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="highTorqueDurationMs"),
        pydantic.Field(
            alias="highTorqueDurationMs",
            description="Time the vehicle engine torque is greater than 90% in milliseconds. ",
        ),
    ] = None
    idling_duration_ms: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="idlingDurationMs"),
        pydantic.Field(alias="idlingDurationMs", description="Time spent idling in milliseconds."),
    ]
    over_speed_duration_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="overSpeedDurationMs"),
        pydantic.Field(alias="overSpeedDurationMs", description="Time spent over-speeding in milliseconds."),
    ] = None
    total_brake_duration_ms: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="totalBrakeDurationMs"),
        pydantic.Field(alias="totalBrakeDurationMs", description="Total breaking time in milliseconds."),
    ]
    total_brake_event_count: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="totalBrakeEventCount"),
        pydantic.Field(alias="totalBrakeEventCount", description="Total number of brake events."),
    ] = None
    wear_free_brake_duration_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="wearFreeBrakeDurationMs"),
        pydantic.Field(alias="wearFreeBrakeDurationMs", description="Time spent ware-free breaking in milliseconds."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

WorkerSafetySosDataResponseBodySourceType = typing.Union[
    typing.Literal["appSos", "noResponseCheckIn", "wearableSos", "fallDetected"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .notification_recipient_request_body_notification_types_item import (
    NotificationRecipientRequestBodyNotificationTypesItem,
)


class NotificationRecipientRequestBody(UniversalBaseModel):
    """
    A user to notify about recovery state changes.
    """

    notification_types: typing.List[NotificationRecipientRequestBodyNotificationTypesItem] = pydantic.Field()
    """
    How the user should be notified.
    """

    user_id: int = pydantic.Field()
    """
    The ID of the user to notify. Users can be retrieved via the getUser API endpoint.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .driver_with_external_id_object_response_body import DriverWithExternalIdObjectResponseBody


class CoachAssignmentWithDriverExternalIdsResponseResponseBody(UniversalBaseModel):
    """
    Driver coach assignment object.
    """

    coach_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="coachId"),
        pydantic.Field(alias="coachId", description="Coach ID associated with coach assignment. Always returned."),
    ]
    created_at_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(alias="createdAtTime", description="Time coach assignment was created in UTC. Always returned."),
    ]
    driver: DriverWithExternalIdObjectResponseBody
    updated_at_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="updatedAtTime"),
        pydantic.Field(
            alias="updatedAtTime", description="Time coaching assignment was updated in UTC. Always returned."
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .v_1_vision_step_results_item_angle_check_angle_configured import V1VisionStepResultsItemAngleCheckAngleConfigured


class V1VisionStepResultsItemAngleCheck(UniversalBaseModel):
    angle_configured: typing_extensions.Annotated[
        typing.Optional[V1VisionStepResultsItemAngleCheckAngleConfigured],
        FieldMetadata(alias="angleConfigured"),
        pydantic.Field(alias="angleConfigured", description="The configured angle allowance range (in degrees)"),
    ] = None
    angle_found: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="angleFound"),
        pydantic.Field(
            alias="angleFound",
            description="The counter-clockwise angle detected from the first edge to the second edge",
        ),
    ] = None
    end_step_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="endStepName"),
        pydantic.Field(
            alias="endStepName", description="The name of the second reference step used to check the angle"
        ),
    ] = None
    start_step_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="startStepName"),
        pydantic.Field(
            alias="startStepName", description="The name of the first reference step used to check the angle"
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class PartInstanceObjectResponseBody(UniversalBaseModel):
    """
    Work Order Part Instance object.
    """

    cost_cents_override: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="costCentsOverride"),
        pydantic.Field(
            alias="costCentsOverride",
            description="The cost of one unit of the part in cents. If not provided, the part defined cost will be used.",
        ),
    ] = None
    id: str = pydantic.Field()
    """
    ID of the part instance.
    """

    part_id: typing_extensions.Annotated[
        str, FieldMetadata(alias="partId"), pydantic.Field(alias="partId", description="ID of the part definition.")
    ]
    quantity: float = pydantic.Field()
    """
    The quantity of the part in the parts pre defined unit of measure.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .safety_event_behavior_label import SafetyEventBehaviorLabel

SafetyEventBehaviorLabels = typing.List[SafetyEventBehaviorLabel]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .defect_types_response_data_response_body_section_type import DefectTypesResponseDataResponseBodySectionType
from .defect_types_response_data_response_body_severity import DefectTypesResponseDataResponseBodySeverity


class DefectTypesResponseDataResponseBody(UniversalBaseModel):
    """
    Defect type data.
    """

    created_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(alias="createdAtTime", description="Time when defect type was created in RFC 3339 format."),
    ]
    id: str = pydantic.Field()
    """
    The unique ID of the DVIR defect type.
    """

    label: str = pydantic.Field()
    """
    DVIR defect type label.
    """

    section_type: typing_extensions.Annotated[
        DefectTypesResponseDataResponseBodySectionType,
        FieldMetadata(alias="sectionType"),
        pydantic.Field(
            alias="sectionType",
            description="Section for DVIR defect type.  Valid values: `exteriorFront`, `exteriorRear`, `exteriorSideUnderneath`, `interiorDriverCab`, `interiorPassengerArea`, `other`, `unknown`",
        ),
    ]
    severity: typing.Optional[DefectTypesResponseDataResponseBodySeverity] = pydantic.Field(default=None)
    """
    The severity of the defect type. Only present for DVIR 2.0 defect types.  Valid values: `major`, `minor`, `notApplicable`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

OperationalSettingsObjectRequestBodyTimeRangeType = typing.Union[
    typing.Literal["activeBetween", "inactiveBetween"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class HosEldEventRemarkObjectResponseBody(UniversalBaseModel):
    comment: str = pydantic.Field()
    """
    The content of the remark.
    """

    location_description: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="locationDescription"),
        pydantic.Field(alias="locationDescription", description="The location description entered by the user"),
    ]
    time: str = pydantic.Field()
    """
    The time in RFC 3339 format at which the remark was created.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

VehicleHarshAccelerationSettingType = typing.Union[
    typing.Literal["passengerCar", "lightTruck", "heavyDuty", "off", "automatic"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .forms_media_record_object_response_body import FormsMediaRecordObjectResponseBody


class FormsSignatureValueObjectResponseBody(UniversalBaseModel):
    """
    The value of a signature form input field.
    """

    media: FormsMediaRecordObjectResponseBody

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class V1VisionStepResultsItemContour(UniversalBaseModel):
    angle_degrees: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="angleDegrees"),
        pydantic.Field(alias="angleDegrees", description="The rotation angle found"),
    ] = None
    angle_tolerance: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="angleTolerance"),
        pydantic.Field(alias="angleTolerance", description="The rotation angle allowance"),
    ] = None
    match_percentage: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="matchPercentage"),
        pydantic.Field(alias="matchPercentage", description="The contour match percentage with the configured contour"),
    ] = None
    match_threshold: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="matchThreshold"),
        pydantic.Field(alias="matchThreshold", description="The configured match threshold for contours"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .notification_recipient_response_response_body_notification_types_item import (
    NotificationRecipientResponseResponseBodyNotificationTypesItem,
)


class NotificationRecipientResponseResponseBody(UniversalBaseModel):
    """
    A user subscribed to recovery state change notifications, including their name and email.
    """

    email: str = pydantic.Field()
    """
    The email address of the user.
    """

    name: str = pydantic.Field()
    """
    The display name of the user.
    """

    notification_types: typing.List[NotificationRecipientResponseResponseBodyNotificationTypesItem] = pydantic.Field()
    """
    How the user is notified.
    """

    user_id: int = pydantic.Field()
    """
    The ID of the user.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class VehicleStatsTirePressures(UniversalBaseModel):
    """
    Tire pressure readings for each of four tires in kilopascals.
    """

    back_left_tire_pressure_k_pa: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="backLeftTirePressureKPa"),
        pydantic.Field(
            alias="backLeftTirePressureKPa",
            description="The tire pressure of the rear left tire as seen when standing behind the vehicle in kilopascals.",
        ),
    ] = None
    back_right_tire_pressure_k_pa: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="backRightTirePressureKPa"),
        pydantic.Field(
            alias="backRightTirePressureKPa",
            description="The tire pressure of the rear right tire as seen when standing behind the vehicle in kilopascals.",
        ),
    ] = None
    front_left_tire_pressure_k_pa: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="frontLeftTirePressureKPa"),
        pydantic.Field(
            alias="frontLeftTirePressureKPa",
            description="The tire pressure of the front left tire as seen when standing behind the vehicle in kilopascals.",
        ),
    ] = None
    front_right_tire_pressure_k_pa: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="frontRightTirePressureKPa"),
        pydantic.Field(
            alias="frontRightTirePressureKPa",
            description="The tire pressure of the front right tire as seen when standing behind the vehicle in kilopascals.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class ReadingHistoryResponseBody(UniversalBaseModel):
    """
    A history of reading values for an entity.
    """

    entity_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="entityId"),
        pydantic.Field(alias="entityId", description="The ID of the entity this readings is for."),
    ]
    external_ids: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]],
        FieldMetadata(alias="externalIds"),
        pydantic.Field(alias="externalIds", description="A map of external ids"),
    ] = None
    happened_at_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="happenedAtTime"),
        pydantic.Field(
            alias="happenedAtTime", description="The time in RFC 3339 format when the reading was measured."
        ),
    ] = None
    value: typing.Optional[typing.Dict[str, typing.Any]] = pydantic.Field(default=None)
    """
    The value of the reading.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class PendingDutyStatusDurationsObjectResponseBody(UniversalBaseModel):
    """
    What the duty status durations on the driver’s log would be if all pending carrier edits are accepted by the driver.
    """

    active_duration_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="activeDurationMs"),
        pydantic.Field(
            alias="activeDurationMs",
            description="Duration the driver was active for in the log period in milliseconds.",
        ),
    ] = None
    drive_duration_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="driveDurationMs"),
        pydantic.Field(
            alias="driveDurationMs",
            description="Duration the driver was driving for in the log period in milliseconds.",
        ),
    ] = None
    off_duty_duration_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="offDutyDurationMs"),
        pydantic.Field(
            alias="offDutyDurationMs",
            description="Duration the driver was off duty for in the log period in milliseconds.",
        ),
    ] = None
    on_duty_duration_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="onDutyDurationMs"),
        pydantic.Field(
            alias="onDutyDurationMs",
            description="Duration the driver was on duty for in the log period in milliseconds.",
        ),
    ] = None
    personal_conveyance_duration_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="personalConveyanceDurationMs"),
        pydantic.Field(
            alias="personalConveyanceDurationMs",
            description="Duration the driver was driving for personal conveyance for in the log period in milliseconds.",
        ),
    ] = None
    sleeper_berth_duration_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="sleeperBerthDurationMs"),
        pydantic.Field(
            alias="sleeperBerthDurationMs",
            description="Duration the driver was in their sleeper berth for in the log period in milliseconds.",
        ),
    ] = None
    waiting_time_duration_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="waitingTimeDurationMs"),
        pydantic.Field(
            alias="waitingTimeDurationMs",
            description="Duration the driver was waiting for in the log period in milliseconds.",
        ),
    ] = None
    yard_move_duration_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="yardMoveDurationMs"),
        pydantic.Field(
            alias="yardMoveDurationMs",
            description="Duration the driver was driving for yard moves for in the log period in milliseconds.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

VehicleResponseObjectResponseBodyVehicleRegulationMode = typing.Union[
    typing.Literal["regulated", "unregulated"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .order_task_custom_property_object_response_body import OrderTaskCustomPropertyObjectResponseBody
from .order_task_skill_object_response_body import OrderTaskSkillObjectResponseBody
from .quantity_object_response_body import QuantityObjectResponseBody


class OrderTaskObjectResponseBody(UniversalBaseModel):
    """
    Order task object with detailed information
    """

    custom_properties: typing_extensions.Annotated[
        typing.Optional[typing.List[OrderTaskCustomPropertyObjectResponseBody]],
        FieldMetadata(alias="customProperties"),
        pydantic.Field(alias="customProperties", description="List of custom properties for the order"),
    ] = None
    id: str = pydantic.Field()
    """
    The order identifier
    """

    quantities: typing.Optional[typing.List[QuantityObjectResponseBody]] = pydantic.Field(default=None)
    """
    List of quantity information for the order
    """

    required_skills: typing_extensions.Annotated[
        typing.Optional[typing.List[OrderTaskSkillObjectResponseBody]],
        FieldMetadata(alias="requiredSkills"),
        pydantic.Field(alias="requiredSkills", description="List of required skills for the order"),
    ] = None
    service_window: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="serviceWindow"),
        pydantic.Field(alias="serviceWindow", description="The service window time range"),
    ] = None
    type: str = pydantic.Field()
    """
    The task type (pickup or delivery)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..types.inline_response_2009 import InlineResponse2009
from ..types.v_1_cargo_response import V1CargoResponse
from ..types.v_1_door_response import V1DoorResponse
from ..types.v_1_humidity_response import V1HumidityResponse
from ..types.v_1_sensor_history_response import V1SensorHistoryResponse
from ..types.v_1_sensors_history_series import V1SensorsHistorySeries
from ..types.v_1_temperature_response import V1TemperatureResponse
from .types.inline_object_6_fill_missing import InlineObject6FillMissing

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawSensorsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def v_1_get_sensors_cargo(
        self, *, sensors: typing.Sequence[int], request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[V1CargoResponse]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Get cargo monitor status (empty / full) for requested sensors.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Sensors** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        sensors : typing.Sequence[int]
            List of sensor IDs to query.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[V1CargoResponse]
            List of sensor objects containing the current cargo status reported by each sensor.
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/sensors/cargo",
            method="POST",
            json={
                "sensors": sensors,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    V1CargoResponse,
                    parse_obj_as(
                        type_=V1CargoResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def v_1_get_sensors_door(
        self, *, sensors: typing.Sequence[int], request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[V1DoorResponse]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Get door monitor status (closed / open) for requested sensors.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Sensors** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        sensors : typing.Sequence[int]
            List of sensor IDs to query.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[V1DoorResponse]
            List of sensor objects containing the current door status reported by each sensor.
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/sensors/door",
            method="POST",
            json={
                "sensors": sensors,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    V1DoorResponse,
                    parse_obj_as(
                        type_=V1DoorResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def v_1_get_sensors_history(
        self,
        *,
        end_ms: int,
        series: typing.Sequence[V1SensorsHistorySeries],
        start_ms: int,
        step_ms: int,
        fill_missing: typing.Optional[InlineObject6FillMissing] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[V1SensorHistoryResponse]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Get historical data for specified sensors. This method returns a set of historical data for the specified sensors in the specified time range and at the specified time resolution.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Sensors** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        end_ms : int
            End of the time range, specified in milliseconds UNIX time.

        series : typing.Sequence[V1SensorsHistorySeries]

        start_ms : int
            Beginning of the time range, specified in milliseconds UNIX time.

        step_ms : int
            Time resolution for which data should be returned, in milliseconds. Specifying 3600000 will return data at hour intervals.

        fill_missing : typing.Optional[InlineObject6FillMissing]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[V1SensorHistoryResponse]
            List of results objects, each containing a time and a datapoint for each requested sensor/field pair.
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/sensors/history",
            method="POST",
            json={
                "endMs": end_ms,
                "fillMissing": fill_missing,
                "series": convert_and_respect_annotation_metadata(
                    object_=series, annotation=typing.Sequence[V1SensorsHistorySeries], direction="write"
                ),
                "startMs": start_ms,
                "stepMs": step_ms,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    V1SensorHistoryResponse,
                    parse_obj_as(
                        type_=V1SensorHistoryResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def v_1_get_sensors_humidity(
        self, *, sensors: typing.Sequence[int], request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[V1HumidityResponse]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Get humidity for requested sensors. This method returns the current relative humidity for the requested sensors.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Sensors** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        sensors : typing.Sequence[int]
            List of sensor IDs to query.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[V1HumidityResponse]
            List of sensor objects containing the current humidity reported by each sensor.
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/sensors/humidity",
            method="POST",
            json={
                "sensors": sensors,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    V1HumidityResponse,
                    parse_obj_as(
                        type_=V1HumidityResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def v_1_get_sensors(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[InlineResponse2009]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Get sensor objects. This method returns a list of the sensor objects in the Samsara Cloud and information about them.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Sensors** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[InlineResponse2009]
            List of sensor objects.
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/sensors/list",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    InlineResponse2009,
                    parse_obj_as(
                        type_=InlineResponse2009,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def v_1_get_sensors_temperature(
        self, *, sensors: typing.Sequence[int], request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[V1TemperatureResponse]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Get temperature for requested sensors. This method returns the current ambient temperature (and probe temperature if applicable) for the requested sensors.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Sensors** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        sensors : typing.Sequence[int]
            List of sensor IDs to query.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[V1TemperatureResponse]
            List of sensor objects containing the current temperature reported by each sensor.
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/sensors/temperature",
            method="POST",
            json={
                "sensors": sensors,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    V1TemperatureResponse,
                    parse_obj_as(
                        type_=V1TemperatureResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawSensorsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def v_1_get_sensors_cargo(
        self, *, sensors: typing.Sequence[int], request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[V1CargoResponse]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Get cargo monitor status (empty / full) for requested sensors.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Sensors** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        sensors : typing.Sequence[int]
            List of sensor IDs to query.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[V1CargoResponse]
            List of sensor objects containing the current cargo status reported by each sensor.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/sensors/cargo",
            method="POST",
            json={
                "sensors": sensors,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    V1CargoResponse,
                    parse_obj_as(
                        type_=V1CargoResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def v_1_get_sensors_door(
        self, *, sensors: typing.Sequence[int], request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[V1DoorResponse]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Get door monitor status (closed / open) for requested sensors.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Sensors** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        sensors : typing.Sequence[int]
            List of sensor IDs to query.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[V1DoorResponse]
            List of sensor objects containing the current door status reported by each sensor.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/sensors/door",
            method="POST",
            json={
                "sensors": sensors,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    V1DoorResponse,
                    parse_obj_as(
                        type_=V1DoorResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def v_1_get_sensors_history(
        self,
        *,
        end_ms: int,
        series: typing.Sequence[V1SensorsHistorySeries],
        start_ms: int,
        step_ms: int,
        fill_missing: typing.Optional[InlineObject6FillMissing] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[V1SensorHistoryResponse]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Get historical data for specified sensors. This method returns a set of historical data for the specified sensors in the specified time range and at the specified time resolution.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Sensors** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        end_ms : int
            End of the time range, specified in milliseconds UNIX time.

        series : typing.Sequence[V1SensorsHistorySeries]

        start_ms : int
            Beginning of the time range, specified in milliseconds UNIX time.

        step_ms : int
            Time resolution for which data should be returned, in milliseconds. Specifying 3600000 will return data at hour intervals.

        fill_missing : typing.Optional[InlineObject6FillMissing]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[V1SensorHistoryResponse]
            List of results objects, each containing a time and a datapoint for each requested sensor/field pair.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/sensors/history",
            method="POST",
            json={
                "endMs": end_ms,
                "fillMissing": fill_missing,
                "series": convert_and_respect_annotation_metadata(
                    object_=series, annotation=typing.Sequence[V1SensorsHistorySeries], direction="write"
                ),
                "startMs": start_ms,
                "stepMs": step_ms,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    V1SensorHistoryResponse,
                    parse_obj_as(
                        type_=V1SensorHistoryResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def v_1_get_sensors_humidity(
        self, *, sensors: typing.Sequence[int], request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[V1HumidityResponse]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Get humidity for requested sensors. This method returns the current relative humidity for the requested sensors.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Sensors** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        sensors : typing.Sequence[int]
            List of sensor IDs to query.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[V1HumidityResponse]
            List of sensor objects containing the current humidity reported by each sensor.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/sensors/humidity",
            method="POST",
            json={
                "sensors": sensors,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    V1HumidityResponse,
                    parse_obj_as(
                        type_=V1HumidityResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def v_1_get_sensors(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[InlineResponse2009]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Get sensor objects. This method returns a list of the sensor objects in the Samsara Cloud and information about them.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Sensors** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[InlineResponse2009]
            List of sensor objects.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/sensors/list",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    InlineResponse2009,
                    parse_obj_as(
                        type_=InlineResponse2009,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def v_1_get_sensors_temperature(
        self, *, sensors: typing.Sequence[int], request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[V1TemperatureResponse]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Get temperature for requested sensors. This method returns the current ambient temperature (and probe temperature if applicable) for the requested sensors.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Sensors** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        sensors : typing.Sequence[int]
            List of sensor IDs to query.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[V1TemperatureResponse]
            List of sensor objects containing the current temperature reported by each sensor.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/sensors/temperature",
            method="POST",
            json={
                "sensors": sensors,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    V1TemperatureResponse,
                    parse_obj_as(
                        type_=V1TemperatureResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pagination import AsyncPager, SyncPager
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_gateway_error import BadGatewayError
from ..errors.gateway_timeout_error import GatewayTimeoutError
from ..errors.internal_server_error import InternalServerError
from ..errors.method_not_allowed_error import MethodNotAllowedError
from ..errors.not_found_error import NotFoundError
from ..errors.not_implemented_error import NotImplementedError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.attribute_tiny import AttributeTiny
from ..types.gross_vehicle_weight import GrossVehicleWeight
from ..types.user_identifier_serial import UserIdentifierSerial
from ..types.vehicle_regulation_mode import VehicleRegulationMode
from ..types.vehicle_response import VehicleResponse
from ..types.vehicle_response_object_response_body import VehicleResponseObjectResponseBody
from ..types.vehicle_type import VehicleType
from ..types.vehicles_list_vehicles_response_body import VehiclesListVehiclesResponseBody
from .types.update_vehicle_request_aux_input_type_1 import UpdateVehicleRequestAuxInputType1
from .types.update_vehicle_request_aux_input_type_2 import UpdateVehicleRequestAuxInputType2
from .types.update_vehicle_request_aux_input_type_3 import UpdateVehicleRequestAuxInputType3
from .types.update_vehicle_request_aux_input_type_4 import UpdateVehicleRequestAuxInputType4
from .types.update_vehicle_request_aux_input_type_5 import UpdateVehicleRequestAuxInputType5
from .types.update_vehicle_request_aux_input_type_6 import UpdateVehicleRequestAuxInputType6
from .types.update_vehicle_request_aux_input_type_7 import UpdateVehicleRequestAuxInputType7
from .types.update_vehicle_request_aux_input_type_8 import UpdateVehicleRequestAuxInputType8
from .types.update_vehicle_request_aux_input_type_9 import UpdateVehicleRequestAuxInputType9
from .types.update_vehicle_request_aux_input_type_10 import UpdateVehicleRequestAuxInputType10
from .types.update_vehicle_request_aux_input_type_11 import UpdateVehicleRequestAuxInputType11
from .types.update_vehicle_request_aux_input_type_12 import UpdateVehicleRequestAuxInputType12
from .types.update_vehicle_request_aux_input_type_13 import UpdateVehicleRequestAuxInputType13
from .types.update_vehicle_request_harsh_acceleration_setting_type import (
    UpdateVehicleRequestHarshAccelerationSettingType,
)

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawVehiclesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        tag_ids: typing.Optional[str] = None,
        attribute_value_ids: typing.Optional[str] = None,
        attributes: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        updated_after_time: typing.Optional[str] = None,
        created_after_time: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[VehicleResponseObjectResponseBody, VehiclesListVehiclesResponseBody]:
        """
        Returns a list of all vehicles.

         <b>Rate limit:</b> 25 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Vehicles** under the Vehicles category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        attribute_value_ids : typing.Optional[str]
            A filter on the data based on this comma-separated list of attribute value IDs. Only entities associated with ALL of the referenced values will be returned (i.e. the intersection of the sets of entities with each value). Example: `attributeValueIds=076efac2-83b5-47aa-ba36-18428436dcac,6707b3f0-23b9-4fe3-b7be-11be34aea544`

        attributes : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data to return entities having given attributes using either name-value pair, or range query (only for numeric and date attributes) separated by a comma. Only entities meeting all the conditions will be returned. Example: `attributes=ExampleAttributeName:some_value&attributes=SomeOtherAttr:123&attributes=Length:range(10,20)&attributes=Date:range(2025-01-01,2025-01-31)`

        updated_after_time : typing.Optional[str]
             A filter on data to have an updated at time after or equal to this specified time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        created_after_time : typing.Optional[str]
             A filter on data to have a created at time after or equal to this specified time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[VehicleResponseObjectResponseBody, VehiclesListVehiclesResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/vehicles",
            method="GET",
            params={
                "limit": limit,
                "after": after,
                "parentTagIds": parent_tag_ids,
                "tagIds": tag_ids,
                "attributeValueIds": attribute_value_ids,
                "attributes": attributes,
                "updatedAfterTime": updated_after_time,
                "createdAfterTime": created_after_time,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    VehiclesListVehiclesResponseBody,
                    parse_obj_as(
                        type_=VehiclesListVehiclesResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.data
                _has_next = False
                _get_next = None
                if _parsed_response.pagination is not None:
                    _parsed_next = _parsed_response.pagination.end_cursor
                    _has_next = _parsed_next is not None and _parsed_next != ""
                    _get_next = lambda: self.list(
                        limit=limit,
                        after=_parsed_next,
                        parent_tag_ids=parent_tag_ids,
                        tag_ids=tag_ids,
                        attribute_value_ids=attribute_value_ids,
                        attributes=attributes,
                        updated_after_time=updated_after_time,
                        created_after_time=created_after_time,
                        request_options=request_options,
                    )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[VehicleResponse]:
        """
        Get information about a specific vehicle.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Vehicles** under the Vehicles category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the vehicle. This can either be the Samsara-specified ID, or an external ID. External IDs are customer specified key-value pairs created in the POST or PATCH requests of this resource, or automatically populated by fields on the vehicle. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `maintenanceId:250020`. Automatically populated external IDs are prefixed with `samsara.`. For example, `samsara.vin:1HGBH41JXMN109186`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[VehicleResponse]
            Returns the specified vehicle object.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"fleet/vehicles/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VehicleResponse,
                    parse_obj_as(
                        type_=VehicleResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        id: str,
        *,
        attributes: typing.Optional[typing.Sequence[AttributeTiny]] = OMIT,
        aux_input_type_1: typing.Optional[UpdateVehicleRequestAuxInputType1] = OMIT,
        aux_input_type_10: typing.Optional[UpdateVehicleRequestAuxInputType10] = OMIT,
        aux_input_type_11: typing.Optional[UpdateVehicleRequestAuxInputType11] = OMIT,
        aux_input_type_12: typing.Optional[UpdateVehicleRequestAuxInputType12] = OMIT,
        aux_input_type_13: typing.Optional[UpdateVehicleRequestAuxInputType13] = OMIT,
        aux_input_type_2: typing.Optional[UpdateVehicleRequestAuxInputType2] = OMIT,
        aux_input_type_3: typing.Optional[UpdateVehicleRequestAuxInputType3] = OMIT,
        aux_input_type_4: typing.Optional[UpdateVehicleRequestAuxInputType4] = OMIT,
        aux_input_type_5: typing.Optional[UpdateVehicleRequestAuxInputType5] = OMIT,
        aux_input_type_6: typing.Optional[UpdateVehicleRequestAuxInputType6] = OMIT,
        aux_input_type_7: typing.Optional[UpdateVehicleRequestAuxInputType7] = OMIT,
        aux_input_type_8: typing.Optional[UpdateVehicleRequestAuxInputType8] = OMIT,
        aux_input_type_9: typing.Optional[UpdateVehicleRequestAuxInputType9] = OMIT,
        engine_hours: typing.Optional[int] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        gateway_serial: typing.Optional[UserIdentifierSerial] = OMIT,
        gross_vehicle_weight: typing.Optional[GrossVehicleWeight] = OMIT,
        harsh_acceleration_setting_type: typing.Optional[UpdateVehicleRequestHarshAccelerationSettingType] = OMIT,
        license_plate: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        notes: typing.Optional[str] = OMIT,
        odometer_meters: typing.Optional[int] = OMIT,
        static_assigned_driver_id: typing.Optional[str] = OMIT,
        tag_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        vehicle_regulation_mode: typing.Optional[VehicleRegulationMode] = OMIT,
        vehicle_type: typing.Optional[VehicleType] = OMIT,
        vin: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[VehicleResponse]:
        """
        Updates the given Vehicle object.

        **Note:** Vehicle objects are automatically created when Samsara Vehicle Gateways are installed. You cannot create a Vehicle object via API.

        You are able to *update* many of the fields of a Vehicle.

        **Note**: There are no required fields in the request body, and you only need to provide the fields you wish to update.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Vehicles** under the Vehicles category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the vehicle. This can either be the Samsara-specified ID, or an external ID. External IDs are customer specified key-value pairs created in the POST or PATCH requests of this resource, or automatically populated by fields on the vehicle. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `maintenanceId:250020`. Automatically populated external IDs are prefixed with `samsara.`. For example, `samsara.vin:1HGBH41JXMN109186`.

        attributes : typing.Optional[typing.Sequence[AttributeTiny]]

        aux_input_type_1 : typing.Optional[UpdateVehicleRequestAuxInputType1]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        aux_input_type_10 : typing.Optional[UpdateVehicleRequestAuxInputType10]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        aux_input_type_11 : typing.Optional[UpdateVehicleRequestAuxInputType11]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        aux_input_type_12 : typing.Optional[UpdateVehicleRequestAuxInputType12]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        aux_input_type_13 : typing.Optional[UpdateVehicleRequestAuxInputType13]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        aux_input_type_2 : typing.Optional[UpdateVehicleRequestAuxInputType2]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        aux_input_type_3 : typing.Optional[UpdateVehicleRequestAuxInputType3]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        aux_input_type_4 : typing.Optional[UpdateVehicleRequestAuxInputType4]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        aux_input_type_5 : typing.Optional[UpdateVehicleRequestAuxInputType5]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        aux_input_type_6 : typing.Optional[UpdateVehicleRequestAuxInputType6]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        aux_input_type_7 : typing.Optional[UpdateVehicleRequestAuxInputType7]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        aux_input_type_8 : typing.Optional[UpdateVehicleRequestAuxInputType8]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        aux_input_type_9 : typing.Optional[UpdateVehicleRequestAuxInputType9]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        engine_hours : typing.Optional[int]
            A manual override for the vehicle's engine hours. You may only override a vehicle's engine hours if it cannot be read from on-board diagnostics. When you provide a manual engine hours override, Samsara will begin updating a vehicle's engine hours based on when the Samsara Vehicle Gateway is recieving power or not. Setting the value to 0 will unset the manual engine hours.

        external_ids : typing.Optional[typing.Dict[str, str]]
            The <a href="/docs/external-ids" target="_blank">external IDs</a> for the given object.

        gateway_serial : typing.Optional[UserIdentifierSerial]

        gross_vehicle_weight : typing.Optional[GrossVehicleWeight]

        harsh_acceleration_setting_type : typing.Optional[UpdateVehicleRequestHarshAccelerationSettingType]
            The harsh acceleration setting type. This setting influences the acceleration sensitivity from which a <a href="https://kb.samsara.com/hc/en-us/articles/360043051792-Safety-Event-Overview" target="_blank">harsh event</a> is triggered. **By default**, this setting is inferred by the Samsara Vehicle Gateway from the engine computer, but it may be set or updated through the Samsara Dashboard or the API at any time. If set to `off`, then no acceleration based harsh events are triggered for the vehicle. Valid values: `passengerCar`, `lightTruck`, `heavyDuty`, `off`, `automatic`.

        license_plate : typing.Optional[str]
            The license plate of the Vehicle. **By default**: empty. Can be set or updated through the Samsara Dashboard or the API at any time.

        name : typing.Optional[str]
            The human-readable name of the Vehicle. This is set by a fleet administrator and will appear in both Samsara’s cloud dashboard as well as the Samsara Driver mobile app. **By default**, this name is the serial number of the Samsara Vehicle Gateway. It can be set or updated through the Samsara Dashboard or through the API at any time.

        notes : typing.Optional[str]
            These are generic notes about the Vehicle. Empty by default. Can be set or updated through the Samsara Dashboard or the API at any time.

        odometer_meters : typing.Optional[int]
            A manual override for the vehicle's odometer. You may only override a vehicle's odometer if it cannot be read from on-board diagnostics. When you provide a manual odometer override, Samsara will begin updating a vehicle's odometer using GPS distance traveled since this override was set. See <a href="https://kb.samsara.com/hc/en-us/articles/115005273667" target="_blank">here</a> for more details.

        static_assigned_driver_id : typing.Optional[str]
            ID for the static assigned driver of the vehicle. Setting the value to 0 will unassign the current driver.

        tag_ids : typing.Optional[typing.Sequence[str]]
            An array of IDs of tags to associate with this vehicle. If your access to the API is scoped by one or more tags, this field is required to pass in.

        vehicle_regulation_mode : typing.Optional[VehicleRegulationMode]

        vehicle_type : typing.Optional[VehicleType]

        vin : typing.Optional[str]
            The VIN of the Vehicle. Most of the time, this will be automatically read from the engine computer by the Samsara Vehicle Gateway. It will be empty if it cannot be read. It can be set or updated through the Samsara Dashboard or the API at any time.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[VehicleResponse]
            Returns the specified vehicle object.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"fleet/vehicles/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "attributes": convert_and_respect_annotation_metadata(
                    object_=attributes, annotation=typing.Sequence[AttributeTiny], direction="write"
                ),
                "auxInputType1": aux_input_type_1,
                "auxInputType10": aux_input_type_10,
                "auxInputType11": aux_input_type_11,
                "auxInputType12": aux_input_type_12,
                "auxInputType13": aux_input_type_13,
                "auxInputType2": aux_input_type_2,
                "auxInputType3": aux_input_type_3,
                "auxInputType4": aux_input_type_4,
                "auxInputType5": aux_input_type_5,
                "auxInputType6": aux_input_type_6,
                "auxInputType7": aux_input_type_7,
                "auxInputType8": aux_input_type_8,
                "auxInputType9": aux_input_type_9,
                "engineHours": engine_hours,
                "externalIds": external_ids,
                "gatewaySerial": gateway_serial,
                "grossVehicleWeight": convert_and_respect_annotation_metadata(
                    object_=gross_vehicle_weight, annotation=GrossVehicleWeight, direction="write"
                ),
                "harshAccelerationSettingType": harsh_acceleration_setting_type,
                "licensePlate": license_plate,
                "name": name,
                "notes": notes,
                "odometerMeters": odometer_meters,
                "staticAssignedDriverId": static_assigned_driver_id,
                "tagIds": tag_ids,
                "vehicleRegulationMode": vehicle_regulation_mode,
                "vehicleType": vehicle_type,
                "vin": vin,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VehicleResponse,
                    parse_obj_as(
                        type_=VehicleResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawVehiclesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        tag_ids: typing.Optional[str] = None,
        attribute_value_ids: typing.Optional[str] = None,
        attributes: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        updated_after_time: typing.Optional[str] = None,
        created_after_time: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[VehicleResponseObjectResponseBody, VehiclesListVehiclesResponseBody]:
        """
        Returns a list of all vehicles.

         <b>Rate limit:</b> 25 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Vehicles** under the Vehicles category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        attribute_value_ids : typing.Optional[str]
            A filter on the data based on this comma-separated list of attribute value IDs. Only entities associated with ALL of the referenced values will be returned (i.e. the intersection of the sets of entities with each value). Example: `attributeValueIds=076efac2-83b5-47aa-ba36-18428436dcac,6707b3f0-23b9-4fe3-b7be-11be34aea544`

        attributes : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data to return entities having given attributes using either name-value pair, or range query (only for numeric and date attributes) separated by a comma. Only entities meeting all the conditions will be returned. Example: `attributes=ExampleAttributeName:some_value&attributes=SomeOtherAttr:123&attributes=Length:range(10,20)&attributes=Date:range(2025-01-01,2025-01-31)`

        updated_after_time : typing.Optional[str]
             A filter on data to have an updated at time after or equal to this specified time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        created_after_time : typing.Optional[str]
             A filter on data to have a created at time after or equal to this specified time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[VehicleResponseObjectResponseBody, VehiclesListVehiclesResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/vehicles",
            method="GET",
            params={
                "limit": limit,
                "after": after,
                "parentTagIds": parent_tag_ids,
                "tagIds": tag_ids,
                "attributeValueIds": attribute_value_ids,
                "attributes": attributes,
                "updatedAfterTime": updated_after_time,
                "createdAfterTime": created_after_time,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    VehiclesListVehiclesResponseBody,
                    parse_obj_as(
                        type_=VehiclesListVehiclesResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.data
                _has_next = False
                _get_next = None
                if _parsed_response.pagination is not None:
                    _parsed_next = _parsed_response.pagination.end_cursor
                    _has_next = _parsed_next is not None and _parsed_next != ""

                    async def _get_next():
                        return await self.list(
                            limit=limit,
                            after=_parsed_next,
                            parent_tag_ids=parent_tag_ids,
                            tag_ids=tag_ids,
                            attribute_value_ids=attribute_value_ids,
                            attributes=attributes,
                            updated_after_time=updated_after_time,
                            created_after_time=created_after_time,
                            request_options=request_options,
                        )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[VehicleResponse]:
        """
        Get information about a specific vehicle.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Vehicles** under the Vehicles category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the vehicle. This can either be the Samsara-specified ID, or an external ID. External IDs are customer specified key-value pairs created in the POST or PATCH requests of this resource, or automatically populated by fields on the vehicle. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `maintenanceId:250020`. Automatically populated external IDs are prefixed with `samsara.`. For example, `samsara.vin:1HGBH41JXMN109186`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[VehicleResponse]
            Returns the specified vehicle object.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"fleet/vehicles/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VehicleResponse,
                    parse_obj_as(
                        type_=VehicleResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        id: str,
        *,
        attributes: typing.Optional[typing.Sequence[AttributeTiny]] = OMIT,
        aux_input_type_1: typing.Optional[UpdateVehicleRequestAuxInputType1] = OMIT,
        aux_input_type_10: typing.Optional[UpdateVehicleRequestAuxInputType10] = OMIT,
        aux_input_type_11: typing.Optional[UpdateVehicleRequestAuxInputType11] = OMIT,
        aux_input_type_12: typing.Optional[UpdateVehicleRequestAuxInputType12] = OMIT,
        aux_input_type_13: typing.Optional[UpdateVehicleRequestAuxInputType13] = OMIT,
        aux_input_type_2: typing.Optional[UpdateVehicleRequestAuxInputType2] = OMIT,
        aux_input_type_3: typing.Optional[UpdateVehicleRequestAuxInputType3] = OMIT,
        aux_input_type_4: typing.Optional[UpdateVehicleRequestAuxInputType4] = OMIT,
        aux_input_type_5: typing.Optional[UpdateVehicleRequestAuxInputType5] = OMIT,
        aux_input_type_6: typing.Optional[UpdateVehicleRequestAuxInputType6] = OMIT,
        aux_input_type_7: typing.Optional[UpdateVehicleRequestAuxInputType7] = OMIT,
        aux_input_type_8: typing.Optional[UpdateVehicleRequestAuxInputType8] = OMIT,
        aux_input_type_9: typing.Optional[UpdateVehicleRequestAuxInputType9] = OMIT,
        engine_hours: typing.Optional[int] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        gateway_serial: typing.Optional[UserIdentifierSerial] = OMIT,
        gross_vehicle_weight: typing.Optional[GrossVehicleWeight] = OMIT,
        harsh_acceleration_setting_type: typing.Optional[UpdateVehicleRequestHarshAccelerationSettingType] = OMIT,
        license_plate: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        notes: typing.Optional[str] = OMIT,
        odometer_meters: typing.Optional[int] = OMIT,
        static_assigned_driver_id: typing.Optional[str] = OMIT,
        tag_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        vehicle_regulation_mode: typing.Optional[VehicleRegulationMode] = OMIT,
        vehicle_type: typing.Optional[VehicleType] = OMIT,
        vin: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[VehicleResponse]:
        """
        Updates the given Vehicle object.

        **Note:** Vehicle objects are automatically created when Samsara Vehicle Gateways are installed. You cannot create a Vehicle object via API.

        You are able to *update* many of the fields of a Vehicle.

        **Note**: There are no required fields in the request body, and you only need to provide the fields you wish to update.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Vehicles** under the Vehicles category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the vehicle. This can either be the Samsara-specified ID, or an external ID. External IDs are customer specified key-value pairs created in the POST or PATCH requests of this resource, or automatically populated by fields on the vehicle. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `maintenanceId:250020`. Automatically populated external IDs are prefixed with `samsara.`. For example, `samsara.vin:1HGBH41JXMN109186`.

        attributes : typing.Optional[typing.Sequence[AttributeTiny]]

        aux_input_type_1 : typing.Optional[UpdateVehicleRequestAuxInputType1]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        aux_input_type_10 : typing.Optional[UpdateVehicleRequestAuxInputType10]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        aux_input_type_11 : typing.Optional[UpdateVehicleRequestAuxInputType11]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        aux_input_type_12 : typing.Optional[UpdateVehicleRequestAuxInputType12]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        aux_input_type_13 : typing.Optional[UpdateVehicleRequestAuxInputType13]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        aux_input_type_2 : typing.Optional[UpdateVehicleRequestAuxInputType2]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        aux_input_type_3 : typing.Optional[UpdateVehicleRequestAuxInputType3]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        aux_input_type_4 : typing.Optional[UpdateVehicleRequestAuxInputType4]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        aux_input_type_5 : typing.Optional[UpdateVehicleRequestAuxInputType5]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        aux_input_type_6 : typing.Optional[UpdateVehicleRequestAuxInputType6]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        aux_input_type_7 : typing.Optional[UpdateVehicleRequestAuxInputType7]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        aux_input_type_8 : typing.Optional[UpdateVehicleRequestAuxInputType8]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        aux_input_type_9 : typing.Optional[UpdateVehicleRequestAuxInputType9]
            The type of <a href="https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs" target="_blank">auxiliary input</a> configured for this Vehicle. Once configured, these inputs will generate dynamic, time-series data that will be available to view in the Samsara Dashboard. **By default**: empty. This can be set or updated through the Samsara Dashboard or the API at any time. Inputs 3-13 are only available on gateways with an attached aux expander. Valid values: `None`, `Emergency Lights`, `Emergency Alarm`, `Stop Paddle`, `Power Take-Off`, `Plow`, `Sweeper`, `Salter`, `Reefer`, `Door`, `Boom`, `Auxiliary Engine`, `Generator`, `Eight-Way Lights`, `Panic Button`, `Privacy Button`, `Front Axle Drive`, `Weight Sensor`, `Other`, `Secondary Fuel Source`, `(ECU) Power Take-Off`.

        engine_hours : typing.Optional[int]
            A manual override for the vehicle's engine hours. You may only override a vehicle's engine hours if it cannot be read from on-board diagnostics. When you provide a manual engine hours override, Samsara will begin updating a vehicle's engine hours based on when the Samsara Vehicle Gateway is recieving power or not. Setting the value to 0 will unset the manual engine hours.

        external_ids : typing.Optional[typing.Dict[str, str]]
            The <a href="/docs/external-ids" target="_blank">external IDs</a> for the given object.

        gateway_serial : typing.Optional[UserIdentifierSerial]

        gross_vehicle_weight : typing.Optional[GrossVehicleWeight]

        harsh_acceleration_setting_type : typing.Optional[UpdateVehicleRequestHarshAccelerationSettingType]
            The harsh acceleration setting type. This setting influences the acceleration sensitivity from which a <a href="https://kb.samsara.com/hc/en-us/articles/360043051792-Safety-Event-Overview" target="_blank">harsh event</a> is triggered. **By default**, this setting is inferred by the Samsara Vehicle Gateway from the engine computer, but it may be set or updated through the Samsara Dashboard or the API at any time. If set to `off`, then no acceleration based harsh events are triggered for the vehicle. Valid values: `passengerCar`, `lightTruck`, `heavyDuty`, `off`, `automatic`.

        license_plate : typing.Optional[str]
            The license plate of the Vehicle. **By default**: empty. Can be set or updated through the Samsara Dashboard or the API at any time.

        name : typing.Optional[str]
            The human-readable name of the Vehicle. This is set by a fleet administrator and will appear in both Samsara’s cloud dashboard as well as the Samsara Driver mobile app. **By default**, this name is the serial number of the Samsara Vehicle Gateway. It can be set or updated through the Samsara Dashboard or through the API at any time.

        notes : typing.Optional[str]
            These are generic notes about the Vehicle. Empty by default. Can be set or updated through the Samsara Dashboard or the API at any time.

        odometer_meters : typing.Optional[int]
            A manual override for the vehicle's odometer. You may only override a vehicle's odometer if it cannot be read from on-board diagnostics. When you provide a manual odometer override, Samsara will begin updating a vehicle's odometer using GPS distance traveled since this override was set. See <a href="https://kb.samsara.com/hc/en-us/articles/115005273667" target="_blank">here</a> for more details.

        static_assigned_driver_id : typing.Optional[str]
            ID for the static assigned driver of the vehicle. Setting the value to 0 will unassign the current driver.

        tag_ids : typing.Optional[typing.Sequence[str]]
            An array of IDs of tags to associate with this vehicle. If your access to the API is scoped by one or more tags, this field is required to pass in.

        vehicle_regulation_mode : typing.Optional[VehicleRegulationMode]

        vehicle_type : typing.Optional[VehicleType]

        vin : typing.Optional[str]
            The VIN of the Vehicle. Most of the time, this will be automatically read from the engine computer by the Samsara Vehicle Gateway. It will be empty if it cannot be read. It can be set or updated through the Samsara Dashboard or the API at any time.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[VehicleResponse]
            Returns the specified vehicle object.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"fleet/vehicles/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "attributes": convert_and_respect_annotation_metadata(
                    object_=attributes, annotation=typing.Sequence[AttributeTiny], direction="write"
                ),
                "auxInputType1": aux_input_type_1,
                "auxInputType10": aux_input_type_10,
                "auxInputType11": aux_input_type_11,
                "auxInputType12": aux_input_type_12,
                "auxInputType13": aux_input_type_13,
                "auxInputType2": aux_input_type_2,
                "auxInputType3": aux_input_type_3,
                "auxInputType4": aux_input_type_4,
                "auxInputType5": aux_input_type_5,
                "auxInputType6": aux_input_type_6,
                "auxInputType7": aux_input_type_7,
                "auxInputType8": aux_input_type_8,
                "auxInputType9": aux_input_type_9,
                "engineHours": engine_hours,
                "externalIds": external_ids,
                "gatewaySerial": gateway_serial,
                "grossVehicleWeight": convert_and_respect_annotation_metadata(
                    object_=gross_vehicle_weight, annotation=GrossVehicleWeight, direction="write"
                ),
                "harshAccelerationSettingType": harsh_acceleration_setting_type,
                "licensePlate": license_plate,
                "name": name,
                "notes": notes,
                "odometerMeters": odometer_meters,
                "staticAssignedDriverId": static_assigned_driver_id,
                "tagIds": tag_ids,
                "vehicleRegulationMode": vehicle_regulation_mode,
                "vehicleType": vehicle_type,
                "vin": vin,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VehicleResponse,
                    parse_obj_as(
                        type_=VehicleResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

FormTemplateResponseObjectResponseBodyFormCategory = typing.Union[
    typing.Literal["general", "routing", "fuel"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .carrier_proposed_assignment_driver_all_of_2_external_ids import CarrierProposedAssignmentDriverAllOf2ExternalIds
from .equipment_id import EquipmentId
from .equipment_installed_gateway import EquipmentInstalledGateway
from .equipment_name import EquipmentName
from .tag_tiny_response import TagTinyResponse


class EquipmentObject(UniversalBaseModel):
    """
    An equipment object.
    """

    asset_serial: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="assetSerial"),
        pydantic.Field(alias="assetSerial", description="An equipment identification number."),
    ] = None
    external_ids: typing_extensions.Annotated[
        typing.Optional[CarrierProposedAssignmentDriverAllOf2ExternalIds],
        FieldMetadata(alias="externalIds"),
        pydantic.Field(alias="externalIds"),
    ] = None
    id: EquipmentId
    installed_gateway: typing_extensions.Annotated[
        typing.Optional[EquipmentInstalledGateway],
        FieldMetadata(alias="installedGateway"),
        pydantic.Field(alias="installedGateway"),
    ] = None
    name: typing.Optional[EquipmentName] = None
    notes: typing.Optional[str] = pydantic.Field(default=None)
    """
    Notes about a piece of equipment. Samsara supports a maximum of 255 chars.
    """

    tags: typing.Optional[typing.List[TagTinyResponse]] = pydantic.Field(default=None)
    """
    An array of all tag mini-objects that are associated with the given equipment.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .driver_efficiency_data_object_response_body import DriverEfficiencyDataObjectResponseBody
from .driver_efficiency_difficulty_score_data_object_response_body import (
    DriverEfficiencyDifficultyScoreDataObjectResponseBody,
)
from .driver_efficiency_percentage_data_object_response_body import DriverEfficiencyPercentageDataObjectResponseBody
from .driver_efficiency_raw_data_object_response_body import DriverEfficiencyRawDataObjectResponseBody


class SingleDriverEfficiencyByDriverDataObjectResponseBody(UniversalBaseModel):
    """
    singleDriverEfficiencyByDriverDataObject
    """

    difficulty_score: typing_extensions.Annotated[
        typing.Optional[DriverEfficiencyDifficultyScoreDataObjectResponseBody],
        FieldMetadata(alias="difficultyScore"),
        pydantic.Field(alias="difficultyScore"),
    ] = None
    driver_id: typing_extensions.Annotated[
        str, FieldMetadata(alias="driverId"), pydantic.Field(alias="driverId", description="ID of the driver.")
    ]
    percentage_data: typing_extensions.Annotated[
        typing.Optional[DriverEfficiencyPercentageDataObjectResponseBody],
        FieldMetadata(alias="percentageData"),
        pydantic.Field(alias="percentageData"),
    ] = None
    raw_data: typing_extensions.Annotated[
        typing.Optional[DriverEfficiencyRawDataObjectResponseBody],
        FieldMetadata(alias="rawData"),
        pydantic.Field(alias="rawData"),
    ] = None
    score_data: typing_extensions.Annotated[
        typing.Optional[DriverEfficiencyDataObjectResponseBody],
        FieldMetadata(alias="scoreData"),
        pydantic.Field(alias="scoreData"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from .raw_client import AsyncRawAttributesClient, RawAttributesClient


class AttributesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAttributesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAttributesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAttributesClient
        """
        return self._raw_client

    def update(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.fleet.attributes.update(
            id="id",
        )
        """
        _response = self._raw_client.update(id, request_options=request_options)
        return _response.data


class AsyncAttributesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAttributesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAttributesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAttributesClient
        """
        return self._raw_client

    async def update(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.fleet.attributes.update(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class AssociationResponseBody(UniversalBaseModel):
    """
    An association between a central device (e.g. vehicle gateway) and a peripheral device.
    """

    association_end_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="associationEndTime"),
        pydantic.Field(
            alias="associationEndTime",
            description="The time when this association ended, in RFC 3339 format. Null if the association is still active.",
        ),
    ] = None
    association_start_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="associationStartTime"),
        pydantic.Field(
            alias="associationStartTime", description="The time when this association started, in RFC 3339 format."
        ),
    ]
    central_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="centralId"),
        pydantic.Field(alias="centralId", description="The Samsara ID of the central device in this association."),
    ]
    peripheral_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="peripheralId"),
        pydantic.Field(
            alias="peripheralId", description="The Samsara ID of the peripheral device in this association."
        ),
    ]
    peripheral_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="peripheralName"),
        pydantic.Field(
            alias="peripheralName", description="The human-readable name of the peripheral device, if available."
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class V1Message(UniversalBaseModel):
    driver_id: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="driverId"),
        pydantic.Field(alias="driverId", description="ID of the driver for whom the message is sent to or sent by."),
    ]
    text: str = pydantic.Field()
    """
    The text sent in the message.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .hos_break import HosBreak
from .hos_cycle import HosCycle
from .hos_drive import HosDrive
from .hos_shift import HosShift


class HosClocks(UniversalBaseModel):
    """
    Remaining durations and start times (where applicable) for various HOS rules. See [this page](https://www.samsara.com/fleet/eld-compliance/hours-of-service) for more information on HOS rules.
    """

    break_: typing_extensions.Annotated[
        typing.Optional[HosBreak], FieldMetadata(alias="break"), pydantic.Field(alias="break")
    ] = None
    cycle: typing.Optional[HosCycle] = None
    drive: typing.Optional[HosDrive] = None
    shift: typing.Optional[HosShift] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

FunctionCodePackageResponseBodyStatus = typing.Union[
    typing.Literal["unknown", "pendingUpload", "uploaded", "deployed"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .create_function_request_config_request_body_params import CreateFunctionRequestConfigRequestBodyParams
from .create_function_request_config_request_body_secrets import CreateFunctionRequestConfigRequestBodySecrets
from .function_schedule_request_body import FunctionScheduleRequestBody


class CreateFunctionRequestConfigRequestBody(UniversalBaseModel):
    """
    Configuration for the new Function.
    """

    handler: str = pydantic.Field()
    """
    The handler entrypoint for the function code (e.g. 'index.handler').
    """

    is_schedule_enabled: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="isScheduleEnabled"),
        pydantic.Field(alias="isScheduleEnabled", description="Whether the run schedule is enabled."),
    ] = None
    params: typing.Optional[CreateFunctionRequestConfigRequestBodyParams] = pydantic.Field(default=None)
    """
    Default parameter values for the function.
    """

    schedule: typing.Optional[FunctionScheduleRequestBody] = None
    secrets: typing.Optional[CreateFunctionRequestConfigRequestBodySecrets] = pydantic.Field(default=None)
    """
    Secrets available to the function at runtime.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class PatchDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody(UniversalBaseModel):
    """
    Metadata about this driver assignment
    """

    source_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="sourceName"),
        pydantic.Field(alias="sourceName", description="Describes where the external assignment is coming from"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class EquipmentHeaderWithAdditionalFieldsResponseBody(UniversalBaseModel):
    """
    Equipment header fields.
    """

    equipment_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="EquipmentID"),
        pydantic.Field(
            alias="EquipmentID",
            description="The unique Samsara ID of the equipment. This is automatically generated when the Equipment object is created. It cannot be changed.",
        ),
    ] = None
    model: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="Model"),
        pydantic.Field(alias="Model", description="The model of the equipment."),
    ] = None
    oem_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="OEMName"),
        pydantic.Field(alias="OEMName", description="The make of the equipment."),
    ] = None
    pin: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="PIN"),
        pydantic.Field(alias="PIN", description="The PIN number of the equipment."),
    ] = None
    serial_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="SerialNumber"),
        pydantic.Field(alias="SerialNumber", description="The serial number of the equipment."),
    ] = None
    unit_install_date_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="UnitInstallDateTime"),
        pydantic.Field(
            alias="UnitInstallDateTime",
            description="Telematics unit install date in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .sensor import Sensor
from .vehicle_sensor_configuration_door_position import VehicleSensorConfigurationDoorPosition


class VehicleSensorConfigurationDoor(UniversalBaseModel):
    """
    A door monitor configuration for a vehicle
    """

    position: typing.Optional[VehicleSensorConfigurationDoorPosition] = pydantic.Field(default=None)
    """
    Position of the door monitor on the vehicle
    """

    sensor: typing.Optional[Sensor] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class FormsScoreObjectResponseBody(UniversalBaseModel):
    """
    Forms score object.
    """

    max_points: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="maxPoints"),
        pydantic.Field(alias="maxPoints", description="Total possible points of the form submission."),
    ]
    score_percent: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="scorePercent"),
        pydantic.Field(
            alias="scorePercent",
            description="Percentage score of the form submission, calculated as scorePoints / maxPoints.",
        ),
    ]
    score_points: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="scorePoints"),
        pydantic.Field(alias="scorePoints", description="Score, in points, of the form submission."),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class SettingsComplianceResponseObjectResponseBody(UniversalBaseModel):
    """
    Information set here will be displayed in roadside inspections and in the transferred US DOT datafile.
    """

    allow_unregulated_vehicles_enabled: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="allowUnregulatedVehiclesEnabled"),
        pydantic.Field(
            alias="allowUnregulatedVehiclesEnabled",
            description="[deprecated] Allow Unregulated Vehicles. This setting is deprecated as all organizations can now mark vehicles as unregulated.",
        ),
    ] = None
    canada_hos_enabled: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="canadaHosEnabled"),
        pydantic.Field(alias="canadaHosEnabled", description="Enable Canada HOS"),
    ] = None
    carrier_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="carrierName"),
        pydantic.Field(alias="carrierName", description="Carrier name of the organization"),
    ] = None
    dot_number: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="dotNumber"),
        pydantic.Field(alias="dotNumber", description="DOT Number"),
    ] = None
    driver_auto_duty_enabled: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="driverAutoDutyEnabled"),
        pydantic.Field(alias="driverAutoDutyEnabled", description="Enable Driver Auto-Duty"),
    ] = None
    edit_certified_logs_enabled: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="editCertifiedLogsEnabled"),
        pydantic.Field(alias="editCertifiedLogsEnabled", description="Drivers Can Edit Certified Log"),
    ] = None
    force_manual_location_for_duty_status_changes_enabled: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="forceManualLocationForDutyStatusChangesEnabled"),
        pydantic.Field(
            alias="forceManualLocationForDutyStatusChangesEnabled",
            description="Force Manual Location For Duty Status Changes",
        ),
    ] = None
    force_review_unassigned_hos_enabled: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="forceReviewUnassignedHosEnabled"),
        pydantic.Field(alias="forceReviewUnassignedHosEnabled", description="Force Review of Unassigned HOS"),
    ] = None
    main_office_formatted_address: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="mainOfficeFormattedAddress"),
        pydantic.Field(alias="mainOfficeFormattedAddress", description="Office Address"),
    ] = None
    persistent_duty_status_enabled: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="persistentDutyStatusEnabled"),
        pydantic.Field(alias="persistentDutyStatusEnabled", description="Persistent Duty Status"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .fuel_energy_vehicle_report_object_response_body import FuelEnergyVehicleReportObjectResponseBody


class FuelEnergyVehicleReportDataObjectResponseBody(UniversalBaseModel):
    """
    Dictionary containing summarized vehicle report data.
    """

    vehicle_reports: typing_extensions.Annotated[
        typing.List[FuelEnergyVehicleReportObjectResponseBody],
        FieldMetadata(alias="vehicleReports"),
        pydantic.Field(alias="vehicleReports", description="List of summarized vehicle reports."),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class V1Sensor(UniversalBaseModel):
    """
    Contains information about a sensor.
    """

    id: int = pydantic.Field()
    """
    ID of the sensor.
    """

    mac_address: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="macAddress"),
        pydantic.Field(alias="macAddress", description="MAC address of the sensor."),
    ] = None
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the sensor.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .carrier_proposed_assignment_trailer import CarrierProposedAssignmentTrailer

CarrierProposedAssignmentTrailers = typing.List[CarrierProposedAssignmentTrailer]
